# flake8: noqa

import warnings

import pytest

try:
    import regex as re  # type: ignore
except ImportError:
    import re  # type: ignore

    warnings.warn(
        "`regex` module not found," " using builtin `re` module", ImportWarning
    )

from remagic import *


@pytest.mark.parametrize(
    "input_str, expected_output",
    [
        ("test", "test"),
        (".", "."),
        (r"a sentence\.", r"a sentence\."),
        (CHAR, "."),
        (CHAR + N, r".\n"),
        (WS * 5, r"\s{5}"),
        (WS * (1, 3), r"\s{1,3}"),
    ],
)
def test_compile_pattern(input_str, expected_output):
    """
    :param input_str: Pattern to compile in remagic.
    :param expected_output: Pattern to compile in regex or re.
    """
    output = create(input_str).compile()
    assert output == re.compile(expected_output), "{} != {}".format(
        output, expected_output
    )



# flake8: noqa

import pytest

from remagic import *


@pytest.mark.parametrize(
    "const, expected_output",
    [
        (CHAR, "."),
        (WS, r"\s"),
        (N, r"\n"),
    ],
)
def test_create_constants(const, expected_output):
    """
    :param const: Constant
    :param expected_output: The expected transformation.
    """
    output = const._pattern
    assert output == expected_output, "{} != {}".format(output, expected_output)


@pytest.mark.parametrize(
    "input_str, expected_output",
    [
        ("test", "test"),
        (".", "."),
        (r"a sentence\.", r"a sentence\."),
        (CHAR, "."),
    ],
)
def test_create_pattern(input_str, expected_output):
    """
    :param input_str: String to transform.
    :param expected_output: The expected transformation.
    """
    output = create(input_str)._pattern
    assert output == expected_output, "{} != {}".format(output, expected_output)


@pytest.mark.parametrize(
    "input_str1, input_str2, expected_output",
    [
        ("test", "test", "testtest"),
        (".", ".", ".."),
        (CHAR, ".", r".."),
        (".", str(CHAR), r".."),
        (CHAR, N, r".\n"),
    ],
)
def test_add_patterns_within_create(input_str1, input_str2, expected_output):
    """
    :param input_str1: 1st String to transform.
    :param input_str2: 2nd String to transform.
    :param expected_output: The expected transformation.
    """
    output = create(input_str1 + input_str2)._pattern
    assert output == expected_output, "{} != {}".format(output, expected_output)


@pytest.mark.parametrize(
    "input_str, num, expected_output",
    [
        ("test", 2, "testtest"),
        (CHAR, 2, ".{2}"),
    ],
)
def test_multiply_patterns_within_create(input_str, num: int, expected_output):
    """
    :param input_str: 1st String to transform.
    :param num: multiplier
    :param expected_output: The expected transformation.
    """
    output = create(input_str * num)._pattern
    assert output == expected_output, "{} != {}".format(output, expected_output)


@pytest.mark.parametrize(
    "input_str, num, expected_output",
    [
        ("test", 2, "testtest"),
        (CHAR, 2, ".{2}"),
        (WS, (2, 5), r"\s{2,5}"),
        (DIGIT, (0,), r"\d*"),
        (DIGIT, (1,), r"\d+"),
        (DIGIT, (0, 1), r"\d?"),
    ],
)
def test_repeat(input_str, num, expected_output):
    """
    :param input_str: 1st String to transform.
    :param num: multiplier
    :param expected_output: The expected transformation.
    """
    output = create(input_str * num)._pattern
    assert output == expected_output, "{} != {}".format(output, expected_output)


@pytest.mark.parametrize(
    "func, greedy, expected_output",
    [
        (zero_or_more, True, r"\d*"),
        (one_or_more, True, r"\d+"),
        (zero_or_more, False, r"\d*?"),
        (one_or_more, False, r"\d+?"),
    ],
)
def test_greedy_quantifier(func, greedy, expected_output):
    """
    :param func: function
    :param greedy: bool, True if greedy
    :param expected_output: The expected transformation.
    """
    output = func(DIGIT, greedy=greedy)._pattern
    assert output == expected_output, "{} != {}".format(output, expected_output)



from enum import Enum
from typing import Union


class Consts(Enum):
    CHAR = r"."
    WHITESPACE = r"\s"
    NOT_WHITESPACE = r"\S"
    WS = r"\s"
    NOT_WS = r"\S"
    WORD = r"\w"
    NOT_WORD = r"\W"
    DIGIT = r"\d"
    NOT_DIGIT = r"\D"
    LETTER = r"[a-zA-Z]"
    NOT_LETTER = r"[^a-zA-Z]"
    TAB = r"\t"
    NEWLINE = r"\n"
    NOT_NEWLINE = r"\N"
    N = r"\n"
    NOT_N = r"\N"
    CARRIAGE_RETURN = r"\r"
    R = r"\r"

    def __add__(self, other: Union["Consts", str]):
        if isinstance(other, Consts):
            return self.value + other.value
        return self.value + str(other)

    def __mul__(self, other: int):
        return self.value + f"{{{int(other)}}}"



"""
Implements RegEx abstraction logic, for easy working with RegEx in Python.
"""
__title__ = "remagic"
__version__ = "0.1.0"
__author__ = "ificiana"
__license__ = "MIT License"
__copyright__ = "Copyright 2022 ificiana"

from .constants import Consts
from .interface import (
    create,
    optional,
    any_of,
    char_in,
    char_not_in,
    exactly,
    zero_or_more,
    one_or_more,
)

CHAR = create(Consts.CHAR)
WHITESPACE = create(Consts.WHITESPACE)
NOT_WHITESPACE = create(Consts.NOT_WHITESPACE)
WS = create(Consts.WS)
NOT_WS = create(Consts.NOT_WS)
WORD = create(Consts.WORD)
NOT_WORD = create(Consts.NOT_WORD)
DIGIT = create(Consts.DIGIT)
NOT_DIGIT = create(Consts.NOT_DIGIT)
LETTER = create(Consts.LETTER)
NOT_LETTER = create(Consts.NOT_LETTER)
TAB = create(Consts.TAB)
NEWLINE = create(Consts.NEWLINE)
NOT_NEWLINE = create(Consts.NOT_NEWLINE)
CARRIAGE_RETURN = create(Consts.CARRIAGE_RETURN)
N = create(Consts.N)
NOT_N = create(Consts.NOT_N)
R = create(Consts.R)

__all__ = [
    "CARRIAGE_RETURN",
    "CHAR",
    "Consts",
    "DIGIT",
    "LETTER",
    "N",
    "NEWLINE",
    "NOT_DIGIT",
    "NOT_LETTER",
    "NOT_N",
    "NOT_NEWLINE",
    "NOT_WHITESPACE",
    "NOT_WORD",
    "NOT_WS",
    "R",
    "TAB",
    "WHITESPACE",
    "WORD",
    "WS",
    "any_of",
    "char_in",
    "char_not_in",
    "create",
    "exactly",
    "one_or_more",
    "optional",
    "zero_or_more",
]



import warnings
from typing import Tuple, Union

from .constants import Consts

try:
    import regex as re  # type: ignore
except ImportError:
    import re  # type: ignore

    warnings.warn("`regex` module not found, using builtin `re` module", ImportWarning)


class Pattern:
    _pattern: str

    def __init__(self, pattern=""):
        self._pattern = pattern

    def create_from_consts(self, const: Consts) -> "Pattern":
        self._pattern = const.value
        return self

    def compile(self) -> "re.Pattern":
        return re.compile(self._pattern)

    def add(self, other: Union[str, "Pattern"]):
        """
        Function to add two Patterns
        :param other: Pattern
        :return: other Pattern appended to this Pattern
        """
        return self + other

    def repeat(self, num: Union[int, Tuple[int, int]]) -> "Pattern":
        return self * num

    @property
    def pattern(self) -> str:
        return self._pattern

    @pattern.getter
    def pattern(self) -> str:
        return self._pattern

    @pattern.setter
    def pattern(self, value):
        self._pattern = value

    def __add__(self, other: Union["Pattern", str]) -> "Pattern":
        if isinstance(other, Pattern):
            return Pattern(self._pattern + other._pattern)
        return Pattern(self._pattern + str(other))

    def __mul__(self, num: Union[int, Tuple[int, int], Tuple[int]]) -> "Pattern":
        if isinstance(num, tuple):
            first, second = num[0], num[1] if len(num) > 1 else None
            if first == 0 and second is None:
                return Pattern(self._pattern + "*")
            if first == 1 and second is None:
                return Pattern(self._pattern + "+")
            if first == 0 and second == 1:
                return Pattern(self._pattern + "?")
            if first == second:
                return Pattern(self._pattern + f"{{{int(first)}}}")
            return Pattern(self._pattern + f"{{{int(num[0])},{int(num[1])}}}")
        return Pattern(self._pattern + f"{{{int(num)}}}")

    def __eq__(self, other):
        return self._pattern == str(other)

    def __iter__(self):
        return iter(self._pattern)

    def __str__(self) -> str:
        return str(self._pattern)



from .constants import Consts
from .pattern import Pattern
from .utils import char_set_escape, escape


def create(pattern):
    if isinstance(pattern, Pattern):
        return pattern
    if isinstance(pattern, Consts):
        return Pattern().create_from_consts(pattern)
    return exactly(pattern)


def exactly(pattern):
    return Pattern(pattern)


def optional(pattern):
    return Pattern(pattern) * (0, 1)


def zero_or_more(pattern, greedy=True):
    return Pattern(pattern) * (0,) if greedy else optional(Pattern(pattern) * (0,))


def one_or_more(pattern, greedy=True):
    return Pattern(pattern) * (1,) if greedy else optional(Pattern(pattern) * (1,))


def _char_set_items(iterable):
    # ordered set of items
    return dict.fromkeys(list("".join([char_set_escape(i) for i in iterable])))


def _alt_items(iterable):
    # ordered set of items
    return dict.fromkeys([escape(i) for i in iterable])


def char_in(iterable):
    return exactly(f"[{''.join(_char_set_items(iterable))}]")


def char_not_in(iterable):
    return exactly(f"[^{''.join(_char_set_items(iterable))}]")


def any_of(iterable):
    return exactly(f"{'|'.join(_alt_items(iterable))}")



# flake8: noqa

import pytest

from remagic import *


@pytest.mark.parametrize(
    "iterable, expected_output",
    [
        ("test", "[tes]"),
        (".", r"[.]"),
        (["a", "e", "i", "o", "u"], "[aeiou]"),
        ({1: "one", 2: "two"}, "[12]"),
        ({1: "one", 2: "two"}.values(), "[onetw]"),
    ],
)
def test_char_in(iterable, expected_output):
    output = char_in(iterable)
    assert output._pattern == expected_output, "{} != {}".format(
        output, expected_output
    )


@pytest.mark.parametrize(
    "iterable, expected_output",
    [
        ("test", "[^tes]"),
        (".", r"[^.]"),
        (["a", "e", "i", "o", "u"], "[^aeiou]"),
        ({1: "one", 2: "two"}, "[^12]"),
        ({1: "one", 2: "two"}.values(), "[^onetw]"),
    ],
)
def test_char_not_in(iterable, expected_output):
    output = char_not_in(iterable)
    assert output._pattern == expected_output, "{} != {}".format(
        output, expected_output
    )


@pytest.mark.parametrize(
    "iterable, expected_output",
    [
        ("test", "t|e|s"),
        (".", r"\."),
        (["a", "e", "i", "o", "u"], "a|e|i|o|u"),
        ({1: "one", 2: "two"}, "1|2"),
        ({1: "one", 2: "two"}.values(), "one|two"),
    ],
)
def test_char_in(iterable, expected_output):
    output = any_of(iterable)
    assert output._pattern == expected_output, "{} != {}".format(
        output, expected_output
    )



# flake8: noqa

import pytest

from remagic import *


@pytest.mark.parametrize(
    "input_pattern1, input_pattern2, expected_output",
    [
        ("test", "test", "testtest"),
        (".", ".", ".."),
        (CHAR, ".", ".."),
        (CHAR, WS, r".\s"),
        ("test", CHAR, "test."),
    ],
)
def test_pattern_addition(input_pattern1, input_pattern2, expected_output):
    """
    :param input_pattern1: 1st pattern.
    :param input_pattern2: 2nd pattern.
    :param expected_output: The expected transformation.
    """
    output = create(input_pattern1) + create(input_pattern2)
    assert output._pattern == expected_output, "{} != {}".format(
        output, expected_output
    )



"""This module contains setup instructions for remagic."""
import codecs
import os

import setuptools  # type: ignore

here = os.path.abspath(os.path.dirname(__file__))

with codecs.open(os.path.join(here, "README.md"), encoding="utf-8") as fh:
    long_description = "\n" + fh.read()

setuptools.setup(
    name="remagic",
    version="0.1.0",
    author="Ificiana",
    author_email="ificiana@gmail.com",
    packages=["remagic"],
    package_data={"": ["LICENSE"], "remagic": ["py.typed", "constants.pyi"]},
    url="https://github.com/ificiana/remagic",
    license="MIT License",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
    ],
    description=(
        "Work with RegEx with relative ease."
        " Partly inspired by magic-regexp for Node"
    ),
    include_package_data=True,
    long_description_content_type="text/markdown",
    long_description=long_description,
    keywords=["remagic", "regex"],
)



class Consts:
    @property
    def value(self) -> str: ...

    CHAR: str
    WHITESPACE: str
    NOT_WHITESPACE: str
    WS: str
    NOT_WS: str
    WORD: str
    NOT_WORD: str
    DIGIT: str
    NOT_DIGIT: str
    LETTER: str
    NOT_LETTER: str
    TAB: str
    NEWLINE: str
    NOT_NEWLINE: str
    CARRIAGE_RETURN: str
    N: str
    NOT_N: str
    R: str
